@@ -71,10 +71,18 @@ static int tree_sort(sqfs_tree_node_t *root)
 	for (it = root->children; it->next != NULL; it = it->next) {
 		if (strcmp((const char *)it->name,
 			   (const char *)it->next->name) == 0) {
-			char *path = sqfs_tree_node_get_path(it);
+			char *path;
+			int ret;
 
-			fprintf(stderr, "Entry '%s' found more than once!\n",
-				path);
+			ret = sqfs_tree_node_get_path(it, &path);
+
+			if (ret == 0) {
+				fprintf(stderr,
+					"Entry '%s' found more than once!\n",
+					path);
+			} else {
+				fputs("Entry found more than once!\n", stderr);
+			}
 
 			sqfs_free(path);
 			return -1;
